/*
 * d_copyhdr.c
 *
 * $Source: /nfs/source/MASTER/garden/datlib/RCS/d_copyhdr.c,v $
 * $Date: 94/01/07 21:58:54 $
 * $Author: sbeck $
 * $Locker:  $
 * 
 * This file is part of the dat library.
 * $Log:	d_copyhdr.c,v $
 * Revision 1.7  94/01/07  21:58:54  sbeck
 * Removed all evidence of floating point representation
 * since it is now completely handled by proper byte order
 * processing. Also changed usage of the predefined macro,
 * MACH_BYTE_ORDER to the function call mach_get_byte_order()
 * which automatically determines the endian-ness of the host
 * machine.
 * 
 * Revision 1.6  1992/12/24  20:41:38  holtzman
 * Made type mismatches generate working rather than bomb.
 *
 * Revision 1.5  91/11/02  23:31:18  klee
 * updated for const declarations
 * 
 * Revision 1.4  90/12/08  03:11:14  kaboom
 * Placed under RCS again, move copyright message to copyright.h.
 * 
 * Revision 1.3  90/07/19  10:26:38  pizza
 * Added dat architecture flag.
 * 
 * Revision 1.2  89/01/13  02:43:33  jwang
 * Add M.I.T. copyright message.
 * 
 * Revision 1.1  85/06/29  00:09:46  musicus
 * Initial revision
 * 
 *
 * #include <dat.h>
 * 
 * int d_copyhdr(dout,din)
 * 
 *    Copies the header from the open dat file "din" to the open dat file
 *    "dout".  Note that this routine only affects
 *    the in-core version of the header, and does not modify any files.
 *    d_close(dout) will have to be called to make the header permanent.
 * 
 *    Inputs:
 *       din - DATFILE pointer for input dat file whose header is to be copied
 *       dout - DATFILE pointer for output dat file getting new header
 *    Returns value:
 *       0 if OK
 *       -1 if malloc() failed
 */

#include "copyright.h"
#include <stdio.h>
#include "dat.h"
char *malloc(), *calloc(), *realloc();

int d_copyhdr
  (dout,din)
DATFILE	*dout;
const DATFILE *din;
{
        struct d_token *ihdr, *ohdr, dummy;
	int i, type;
	enum byte_order	bo ;
	extern void perror();
	ihdr = din->db_h;
	ohdr = &dummy;
	if(ihdr == (struct d_token *)-1)
	{
		dout->db_h = (struct d_token *) NULL;
		return(0);
	}

	while(ihdr)
	{
		/* Get space for next token */
		if((ohdr->d_next = (struct d_token*)calloc(1,sizeof(struct d_token)))
					== 0) goto err1;
		ohdr = ohdr->d_next;

		/* copy key string */
		if((ohdr->d_key = malloc((unsigned)(strlen(ihdr->d_key)+1)
					*sizeof(char))) == 0) goto err2;
		strcpy(ohdr->d_key,ihdr->d_key);

		/* copy value string */
		if((ohdr->d_value = malloc((unsigned)(strlen(ihdr->d_value)+1)
					*sizeof(char))) == 0) goto err3;
		strcpy(ohdr->d_value,ihdr->d_value);

		/* advance to next token */
		ihdr = ihdr->d_next;
	}
	dout->db_h = dummy.d_next;				/* Set output token list */
	i = 0;
	goto end;

err2:
	ohdr->d_key = (char*) NULL;
err3:
	ohdr->d_value = (char*) NULL;
err1:
	perror("d_copyhdr");
	i = -1;
end:
	ohdr->d_next = (struct d_token*) NULL;		/* End linked list */
	if(dout->db_h)
		dout->db_flag |= _DB_WRHDR;			/* Modified header flag */



	/* if byte order is non-existent, and we aren't 
	 * creating a new dat file, warn the user. 
	 * if creating a new dat file, set the correct architecture.
	 * if dat file flags conflicts with current machine, warn the user.
	 * if dat file flags and machine agree, do nothing, we're okay.
	 */

	d_flush(dout) ;

	/* get the type of the datfile */
	type = d_gettype(dout, NULL) ;

	/*	printf ("type: %c\n", (char)type) ; */
	
	/* what checks are made depends on the file type */
	switch (type) {
	case 'f':	/* float  */ 
	case 'F':	/* double */
	case 'z':	/* complex float  */
	case 'Z':	/* complex double */
	case NOKEY:	/* unsure of type, check bo */
	case 'd':	/* 2 byte signed integer */
	case 'D':	/* 4 byte signed integer */
	case 'u':	/* 2 byte unsigned integer */
	case 'U':	/* 4 byte unsigned integer */
	  bo = d_get_byte_order(dout) ;
	  if (bo == NOKEY) {
	    /* byte order flag non-existent */
	    d_set_byte_order(dout, mach_get_byte_order()) ;
	  }
	  else {
	    /* byte order flag exists */
	    if (mach_get_byte_order() != bo) {
	      /* fprintf(stderr, "Dat Warning: Byte Order Mismatch.\n") ; */
	      /* fprintf(stderr, "Exiting...\n") ; */
	      /* exit (-1) ; */
		fprintf(stderr,
			"d_copyhdr: output byte order differs from input.\n") ;
		d_set_byte_order(dout, mach_get_byte_order());
	    }
	  }	  
	  break ;
	  
	  /* for the following cases, don't need 
	     to do any architecture checks */
	case 'a':	/* 1 byte ascii */
	case 'v':	/* 1 byte variant */
	case 'c':	/* 1 byte signed integer */
	case 'b':	/* 1 byte unsigned integer */
	default:	/* a type no ones heard of */
	  break ;
	}

	d_flush(dout) ;

	return(i);
}
