/*
 * d_fork.c
 *
 * $Source: /mas/garden/source/MASTER/garden/datlib/RCS/d_fork.c,v $
 * $Date: 228/12/25 00:12:16 $
 * $Author: holtzman $
 * $Locker: klee $
 *
 * This file is part of the dat library.
 *
 * $Log: d_fork.c,v $
 * Revision 1.6  228/12/25  00:12:16  holtzman
 * fixed prototyping error for reopenmode()
 *
 * Revision 1.5  92/06/23  18:38:28  holtzman
 * Fixed aix bug in d_fork -- open mode should always be r+, not a
 * 
 * Revision 1.4  91/11/02  22:34:36  klee
 * found lying around...
 * 
 * Revision 1.3  90/12/08  03:09:38  kaboom
 * Placed under RCS, moved copyright message to copyright.h.
 * 
 */

#include "copyright.h"
#include "dat.h"
static char rcsid[]="$Header: /mas/garden/source/MASTER/garden/datlib/RCS/d_fork.c,v 1.6 228/12/25 00:12:16 holtzman Exp klee $";

/*
int d_fork(in, out, mode)
DATFILE *in, *out[];
int mode;

d_fork splits the DATFILE stream in into a vector of streams out[], each
restricted to a subset of the current submatrix of the parent.  Mode, which is
an integer between 0 and ndims-1 or one of the constants D_ADDR_CHAN (-1) or
D_ADDR_DSET (-2), selects the dimension to split.  For example, use D_ADDR_CHAN
to split a color image sequence into three streams, one channel per color
separation.

The parent stream will be flushed but is otherwise unchanged by d_fork.  Each
child will have an I/O pointer independent from those of other streams.  The
number of children is returned.  */

#ifdef __STDC__
static char *reopenmode(DATFILE *);
#else
static char *reopenmode();
#endif

int 
  d_fork(in, out, d)
DATFILE *in,*out[];
int d;			/* select dimension to split, -2<=d<=ndims-1 */
{
  int i, ndims, nchild;
  char *fmode;
/*  static char *reopenmode();*/
  struct d_addr start, stop, step;
  
  d_flush(in);	/* so that descriptor is available when re-opened */
  
  /* Get parent's submatrix parameters */
  ndims = d_getmatrix(in, &start, &stop, &step);
  
  if (d<-2 || d>=ndims)
    return(_daterrh("d_fork: invalid dimension selected on %s\n",
		    d_getname(in)));
  
  fmode = reopenmode(in);		/* mode for re-opening datfile */
  
  /* dim[-1] and dim[-2] are valid because of the 
   * definition of struct d_addr 
   */
  nchild = (stop.dim[d] - start.dim[d] + step.dim[d] - 1) / step.dim[d];
  
  /* Step through dimension d, opening one datfile for each valid index */
  for (i=0; i<nchild; i++) 
    {
      /* Re-opening datfile may get fooled if chdir() took place since
	 parent was opened. */
      if ((out[i]=d_open(d_getname(in), fmode)) == (DATFILE *)NULL)
	return(_daterrh(""));	/* d_open already printed message */
      stop.dim[d] = start.dim[d]+1;
      if (d_submatrix(out[i], &start, &stop, &step)==-1L)
	return(-1);
      start.dim[d] += step.dim[d];
    }
	
  return(nchild);
}

/* Figure out how to re-open a datfile by determining the mode with which it
 * was originally opened. 
 */

#ifndef linux
static char *
  reopenmode(in)
DATFILE *in;
{
  short flag = d_getfp(in)->_flag;
  
  /* Definitions for flag bits aren't documented anywhere, so the intended
     meanings were guessed from fopen.c. */
  
  if (flag & _IORW)		/* originally opened with r+, w+, or a+ */
    return("r+");	
  else if (flag & _IOREAD)	/* originally opened with r */
    return("r"); 
  else if (flag & _IOWRT)		/* originally opened with a or w */
#ifdef _AIX			/* If aix, never return with "a" */
    return("r+");
#else
    return("a");	/* don't use w, which would clobber existing file */
#endif
  else return(NULL);			/* don't know */
}
#endif /* linux */

#ifdef linux
static char *reopenmode (DATFILE *in) { return NULL; }
#endif /* linux */
