.TH RANGE L datfile
.SH NAME
range \- notation for specifying arbitrary portions of a datfile
.SH SYNOPSIS
[datasets range] [channels range] range range ... 

range ::= [start]:[end][/subsample]
.br
or [start][+size][/subsample]
.SH DESCRIPTION
A standard notation for ranges of a multidimensional datfile, as required by
datfile commands such as
.I pick
and
.IR peek ,
is described.
Arbitrary multidimensional ranges are supported by the 
.B d_submatrix
function of the
.IR datio (L)
library.
.PP
.I range
must be in the form [a][:|+][b][/c].
A range specified in the form 
.I a:b
selects data corresponding to coordinates between 
.I a
and 
.I b,
inclusive.
.I a
and
.I b
must satisfy 0 <= 
.I a, b
<
.I size 
where 
.I size 
is the length of the dataset/channel/dimension being considered.
A range specified using
.I a+b
means
to select data between
.I a
and
.I a+b-1
inclusive.
A series of
multiple range specifications is interpreted from slowest varying dimension to
fastest varying dimension. 
If the user specifies fewer ranges than the dimensionality of the
datfile, 
it is as if the user had specified 0:max_i for the i'th unspecified
dimension. 
For convenience, 
arguments specified as 
.I a:,
.I :,
.I :b,
and
.I +b,
are interpreted as
.I a:end,
.I 0:end,
.I 0:b,
and
.I 0+b.
These abbreviations are useful when one does not remember the size of
a particular channel, dataset, or dimension.
If the user specifies a single number, 
certain commands (such as
.BR pick )
will
select data corresponding to that hyperplane and reduces the
dimension of the datfile.
Finally, if the user specifies
.I /c
as an option, 
it means to
subsample the data by a factor of 
.I c.
.SH EXAMPLES
.PP
To select the first five columns of the top eight rows of the datfile,
the following range specifications are equivalent:
.PP
.B "0:7 0:4"

.B "0+8 0+5"

.B ":7 :4"

.B ":7 0+5"

To select the third channel of all the frames of a moving sequence,
one might use the range:
.PP
.B "chan 2"

Notice that the 
.I datasets
and
.I channels
keywords may be abbreviated to any nonzero subset of the words
"datasets" and "channels", respectively.

To select the even and odd fields of a picture, one might use:
.PP
.B "0:/2"

.B "1:/2"

To select the tenth column of a 2-D datfile and store the result in a
1-D datfile, use:
.PP
.B ": 10"

Notice that in this case, the empty specifier 
.I :
is necessary to indicate that the column is desired. To select the tenth
row of a 2-D datfile, use:
.PP
.B "10"

To subsample the third dimension of a 4-D datfile (in mathematical notation,
subsampling
.I c
of the 4 dimensional system 
.I "crazy[d][c][b][a]"
) while simultaneously leaving the first and second dimensions alone,
one might use:
.PP
.B ": /3"

.SH AUTHOR
John Wang
.SH "SEE ALSO"
datio(L)
.SH BUGS
The getrange routine used to parse ranges should be installed in the library.

