.ds DA DAT\" comment this out if datfile support is not installed
.TH SCANARGV L
.SH NAME
scanargv \- command line argument scanner with scanf-like syntax
.SH SYNOPSIS
.B scanargv(
argc, argv, usage, format
[ , pointer ] . . .
.B )
.br
.B int argc;
.br
.B char *argv[], help[], format[];
.SH TUTORIAL
.I scanargv
does for command line argument lists
what
.IR scanf (3)
does for files.
.PP
.nf
main(argc,argv)
int argc;
char *argv[];
{
	\*(DAFILE *in, *out;
	float yoffset=3.0, xoffset=299.0;
	int verbose=0, rate;
	
	scanargv(argc, argv, "%s in out rate [-verbose] [-o yoff xoff]",
		"%r %w %d [-verbose %+|-o %f %f]",
		&in, &out, &rate, &verbose, &yoffset, &xoffset);
}
.fi
.PP
In the argument list passed to scanargv,
the first string contains documentation printed to
the user if no arguments or inappropriate arguments are typed.
The second string tells how to decode the command line.
.I scanargv
performs the following functions:
.IP 1.
%r and %w direct
.I scanargv
to open files named in
.IR argv [1]
and
.IR argv [2]
for reading and writing, respectively, and to return pointers to them in the
variables
.I in
and
.IR out .
.IP 2.
.IR argv [3]
is converted to integer variable
.IR rate .
.IP 3.
Options, enclosed by [ ] and separated by |, are accepted in any order
starting from
.IR argv [4].
If "-verbose" is seen in the command line, %+ causes the integer variable
.I verbose
to be incremented; otherwise,
.I verbose
is unchanged from its previous value of 0.
.IP 4.
If the option "-o" is seen, the
float variables
.I yoffset
and
.IR xoffset 
will be set from the next 2 arguments;
otherwise, their values are unchanged.
.SH ARGUMENTS
.PP
The
.I scanargv
function returns the number of words successfully processed (including
.IR argv [0]).
This can be used to decide how many input items were found.
.PP
.I argv
is a vector of command line words of length
.IR argc ,
just as received by main().
.PP
.I help
should contain user-oriented documentation.
If the command line is typed incorrectly,
``usage:
.IR help \en'',
is printed to stderr
and the program exits returning the number of
successfully parsed words.
If
.I help
contains one
%s
then
.IR argv [0]
(typically the program name) will be printed there.
If
.I help
is NULL,
.I scanargv
returns with a negative value
instead of printing a message and exiting.
In addition, if the environment variable
VERBOSE_SCANARGV
is set, scanargv will also print the current
command line before exiting.
.PP
The
.I pointer
arguments tell where to store converted results, and are in one-to-one
correspondence from left to right with occurrences of %conversion in
.IR format .
When a conversion (other than %+) is encountered in the
second or later iterations of
a repeatable expression, the pointer is autoincremented to the next location
(see options lists, below).
.PP
Command line words starting from
.IR argv [1]
are processed according to
.IR format ,
an extended version of that used in
.IR scanf .
It
contains a valid
.IR expression ,
which is defined as a (possibly empty) sequence of the following:
.IP 1.
White space (blank, tab, or newline), which separates adjacent patterns.
A tab in the last character of
.IR format ,
however, has a special function: it means that it is not an error
for there to be unmatched command line words remaining after
.I format
is exhausted.
Ordinarily, it is an error for there to be leftovers.
.IP 2.
A non-empty
.I pattern
of the form
.IR string ,
.IR string *,
*,
.RI % conversion ,
or
.IR string % conversion ,
which matches one command line word according to the following rules.
.I string
is a sequence of ordinary characters (not
()[]|%* or white space), which match themselves literally.
* matches zero or more characters, while
.RI % conversion
matches something acceptable to the scanf-like conversion.
(Exception: * in the first character of
.IR format
is ignored, for compatibility with earlier versions of
.IR scanargv ,
in which
parsing began at
.IR argv [0]
and the initial * was used to skip it.)
.IP 3.
An
.I OR list
containing zero or more alternative expressions surrounded
by ( ) and separated by |, which matches exactly one
of the contained expressions.
.IP
The parser follows the leftmost
expression whose first pattern matches the next command line word, 
with no lookahead.
For example, given the OR list ``(%d %f|%d %c)'' and the
command line ``3 z'', the parser matches 3 with %d in the first expression
and gets stuck trying to match z with %f.
Beware that numbers, and strings like -k,
are valid matches for %s or %w, which is usually
not the desired effect; thus, expressions beginning with a
numeric pattern or a literal string should appear earlier in the OR
list.
For example, ``(%d|%s)'' makes more sense than ``(%s|%d)''.
.IP 4.
An
.I options list
containing zero or more optional expressions surrounded
by [ ] and separated by |.
This is similar to an OR list, except it is not an error for 
nothing to be matched and it is possible for several
matches to occur in sequence, in any order.
For example, the options list ``[-x|-y]'' matches ``-x'', ``-y'',
``-x -y'', ``-y -x'', or the empty set.
OR lists
and options lists may be nested arbitrarily.
For example, ``[(-x|-y)]'' denotes mutually exclusive options.
.IP
Each expression is matched at most once unless followed by white space and
a sequence of
.IR k \(>=2
periods, e.g., ``[-x %f .....|-y]'', which makes the first expression
repeatable up to
.IR k =5
times.
The argument corresponding to this %f must point to an array of 5 floats,
which will be stored into consecutively.
The period character has no special meaning outside
the last component of an optional expression.
.SH CONVERSIONS
Unlike scanf,
neither assignment suppression nor maximum field width may be specified.
.TP
.B "%d, %o, %x, %f, %s, %c"
These conversions are almost identical to those of scanf.
Capitalizing the conversion letter or preceding by
l or h has the expected meaning.
Unlike scanf, however, %s does not skip over white space.
.TP
.B %p
A pointer to the command line word is stored through the
the corresponding argument of type (char **).
This is similar to %s, but a pointer is copied instead of the string
itself.
.TP
.B %r, %r+, %w, %w+, %a, %a+
The command line word is accepted as a file name, which is
opened in the indicated mode; a \*(DAFILE pointer is stored through
the corresponding argument of type (\*(DAFILE **).
.if !"\*(DA"" \{\
A bracketed
.IR range (L)
expression appended to the file name, e.g. ``name[ch99,10:20/2,3+45]'',
causes a submatrix of the file to be opened.
The comma separator may be replaced by spaces or ``][''.\}
As long as the
.I help
string is not NULL,
.I scanargv
does not return to the caller if any file cannot be opened; thus,
the calling program does not need to test for NULL file pointers.
To minimize the chance of clobbering existing files if the command
line is mistyped, no output files are opened until
the whole command line is successfully parsed and
all input files are successfully opened.
.TP
.B %+, %++, %+++, ...
Does not match anything, but increments the integer pointed to
by the corresponding argument of type (int *)
as many times as there are + signs.
It is often useful to follow a literal string by %+, to report
the string match to the caller.
Note: The location that %+ affects
is repeatedly incremented by iterations of a repeatable expression, e.g.,
``[-x %+ ....]'' will increment the corresponding variable up to 4 times.
.SH EXAMPLE
.nf
main(argc,argv)	/* silly example */
int argc;
char *argv[];
{
	\*(DATFILE *in, *out;
	char *label, kstr[40];
	static double scalar[4]={1,2,3,4};	/* default values */
	float kfactor1=2.718, kfactor2=3.14;
	int nscalars=0, iflg=0, bs=512;
	
	scanargv(argc, argv,
	"%s in [scalar ...] label out [bs=bs | -k (kf1|kstr) [kf2] | -i]",
	"%r+ [%lf %+ ....] %p %w [bs=%d | -k (%f|%s) [%f]|-i %+]",
		&in,		/* open read/write file */
		scalar,&nscalars,	/* zero to four scalars */
		&label,		/* a string pointer */
		&out,		/* open output file */
		&bs,		/* bs= option: parameter immediately follows keyword */
		&kfactor1, kstr,	/* -k option: numeric or string parameter */
		&kfactor2,			/* optional second parameter */
		&iflg);		/* -i option */
	. . .
}
.fi
.SH "SEE ALSO"
.if !"\*(DA"" dat(L), range(L),
scanf(3)
.SH VERSION
ANSI C Revision 1.0
.SH AUTHOR
yazoo@media-lab.mit.edu
.SH BUGS
.PP
There is no escape mechanism (e.g. %%) to put metacharacters in patterns.
The parser has no lookahead or backtracking capability.
.PP
An optional expression that is used up isn't re-enabled if the enclosing list
is entered again.
For example, if %f is matched on the first pass of ``[x [%f|z] ..]''
it is no longer available on the second pass.
(This might be construed as a feature, since it prevents the memory
corresponding to %f from being overrun.)
.PP
No provision for the order of optional arguments to be significant.
Doesn't support some styles of optional arguments, viz:
.TP 4
1.
multiple parameters in a single word, like ``123:456''
.TP
2.
single key letters which may be concatenated, like ``ls -ut'', and whose
corresponding parameters appear in successive words, like ``tar cbf 20
/dev/mt0''
