/***************************************************************\
*   sndffname.c			MACINTOSH			*
*   functions to keep a table of filenames by filehandles	*
*   THIS FILE IS CONDITIONALLY INCLUDED IN sndfmac.c etc	*
*   12jun91 dpwe						*
\***************************************************************/

#ifndef THINK_C
typedef char Str255[256];
#endif /* !THINK_C */

/* private type declarations */
typedef struct fnameinfo
    {
    struct fnameinfo *next;
    FILE   *fhan;
    Str255 fname;
    long   sndstart;	/* absolute seek points of start and end of sound */
    long   sndend;	/*   .. to allow faking of eof in AIFFs */
    } FNAMEINFO;	/* used to build a map between FILE * & filenames */

/* static variables */
FNAMEINFO   *fniroot = NULL;	/* root of fnameinfo list */

/* static function prototypes */
static char *FindFname PARG((FILE *file));
    /* looks up file ptr, returns mapped name if any */
static void SetSndEnds PARG((FILE *file, long start, long end));
static void GetSndEnds PARG((FILE *file, long *pstart, long *pend));

/* ----------------------- implementations ------------------------ */

static void AddFname(char *fname, FILE *file)
    /* associates name 'fname\0' with file pointer 'file' in our lookup table */
    {
    FNAMEINFO *runner = fniroot;

    /* see if FILE is already in our table.. */
    while(runner != NULL && runner->fhan != file)
	runner = runner->next;
    if(runner == NULL)	    /* not found : add new record */
	{
	runner = (FNAMEINFO *)malloc(sizeof(FNAMEINFO));
	runner->fhan = file;
	runner->next = fniroot;
	runner->sndstart = 0L;
	runner->sndend   = 0L;
	fniroot = runner;	/* add to top of list, faster to find */
	}   /* either way, copy in the (new) name */
    strcpy((char *)runner->fname, fname);
    }

static void RmvFname(FILE *file)
    /* removes any fnameinfo record associated with 'file' */
    {
    FNAMEINFO *node;
    FNAMEINFO **prunner = &fniroot;

    while((*prunner) != NULL && (*prunner)->fhan != file)
	prunner = &(*prunner)->next;
    if((*prunner) != NULL)	/* found : delete record */
	{
	node = (*prunner);
	*prunner = (*prunner)->next;	/* close gap.. */
	free(node);			/* & dispose of memory */
	}
    /* else not found - do nothing */
    }

static char *FindFname(FILE *file)
    /* looks up file ptr, returns mapped name if any */
    {
    FNAMEINFO *runner = fniroot;

    while(runner != NULL && runner->fhan != file)
	runner = runner->next;
    if(runner != NULL)
	return( (char *)runner->fname );
    else
	return( NULL );     /* fileptr not in our list */
    }

static void SetSeekEnds(FILE *file, long start, long end)
    /* register start and end indexes */
    {
    FNAMEINFO *node;
    FNAMEINFO **prunner = &fniroot;

    while((*prunner) != NULL && (*prunner)->fhan != file)
	prunner = &(*prunner)->next;
    if((*prunner) != NULL)	/* found : set vals */
	{
	(*prunner)->sndstart = start;
	(*prunner)->sndend   = end;
	}
    else{	/* else not found - error */
/*	fprintf(stderr, 
		"sndffname:SetSeekEnds: cannot file file %ld (%ld-%ld)\n",
		file, start, end);	*/
	AddFname("<unknown>", file);	/* give it somewhere to go */
	SetSeekEnds(file, start, end);
/*	abort();	*/
	}
    }
    
static void GetSeekEnds(FILE *file, long *pstart, long *pend)
    /* register start and end indexes */
    {
    FNAMEINFO *node;
    FNAMEINFO **prunner = &fniroot;

    while((*prunner) != NULL && (*prunner)->fhan != file)
	prunner = &(*prunner)->next;
    if((*prunner) != NULL)	/* found : set vals */
	{
	*pstart = (*prunner)->sndstart;
	*pend   = (*prunner)->sndend;
	}
    else{	/* else not found - error */
	fprintf(stderr, "sndffname:GetSeekEnds: cannot file file %ld\n",
		file);
	abort();
	}
    }

