/***************************************************************\
*   sndfnat.c			NATIVE				*
*   Header functions for native version of sndf.c		*
*   THIS FILE IS CONDITIONALLY INCLUDED IN sndf.c		*
*   12aug90 dpwe						*
\***************************************************************/

/* exported file type indicator */
char *sndfExtn = ".sdf";		/* file extension, if used */

/* filename table not required : dummy functions to call */
static void AddFname(char *fname, FILE *file)
    {
    }
static void RmvFname(FILE *file)
    {
    }

static int SFReadHdr(FILE *file, SFSTRUCT *snd, int infoBmax)
/* utility to sample the header of an opened file, at most infoBmax info bytes */
    {
    long    infoXtra, bufXtra;
    size_t  num;

    if(infoBmax < SFDFLTBYTS)	infoBmax = SFDFLTBYTS;
    infoBmax = (infoBmax)&-4;	/* round DOWN to x4 (can't exceed given buffer) */
    bufXtra = (long)(infoBmax-SFDFLTBYTS);
    snd->magic = 0L;	snd->headBsize = 0; /* can test for valid read.. */
    if(fseek(file, (long)0, SEEK_SET) != 0 )	/* move to start of file */
	return(SFerror = SFE_RDERR);
    /* read in two stages to make sure no overrun. */
    /* First read minimum header (including header length) */
    num = fread((void *)snd, (size_t)1, (size_t)sizeof(SFSTRUCT), file);
    if(num != sizeof(SFSTRUCT))     /* couldn't get that many bytes */
	return(SFerror = SFE_RDERR);		/* .. call it a read error */
    if(snd->magic != SFMAGIC || snd->headBsize < sizeof(SFSTRUCT))
	return(SFerror = SFE_NSF);	    /* got the bytes, but they look wrong */
    infoXtra = snd->headBsize - sizeof(SFSTRUCT);
    /* if more info is indicated, try & get that too */
    if(infoXtra > 0)
	{
	if(bufXtra > infoXtra)	bufXtra = infoXtra;
	num = fread(((char *)snd)+num, (size_t)1, (size_t)bufXtra, file);
	if( num < bufXtra )
	    return(SFerror = SFE_RDERR);
	}
    return(SFerror = SFE_OK);
    }

int SFRdXInfo(file, psnd, done)
    FILE *file;
    SFSTRUCT *psnd;
    int done;
/* fetch any extra info bytes needed in view of read header
    Passes open file handle, existing sound struct
    already read 'done' bytes; rest inferred from structure */
    {
    size_t  num;
    long    subhlen;
    long    file_pos;
    int     rem;

    rem = SFInfoSize(psnd) - done;
    if(rem <= 0)
	return(SFerror = SFE_OK);	/* nuthin more to read */
    /* do not assume seek already at right place.. */
    subhlen = sizeof(SFSTRUCT) - SFDFLTBYTS + (long)done;
    file_pos = ftell(file);
    if( (subhlen - file_pos) != 0) 	/* if not at right place.. */
	if(fseek(file, subhlen - file_pos, SEEK_CUR) != 0 )	/* relative */
	    return(SFerror = SFE_RDERR);
    rem = SFInfoSize(psnd) - done;
    num = fread((void *)(psnd->info+done), (size_t)1, (size_t)rem, file);
    if(num < (size_t)rem)
	return(SFerror = SFE_RDERR);
    else
	return(SFerror = SFE_OK);
    }

static int SFWriteHdr(FILE *file, SFSTRUCT *snd, int infoBmax)
/*  Utility to write out header to open file
    infoBmax optionally limits the number of info chars to write.
    infoBmax = 0 => write out all bytes implied by .headBsize field
    infoBmax >= SFDFLTBYS => write out only infoBmax infobytes */
    {
    size_t  num;
    long    hedSiz;

    if(snd->magic != SFMAGIC)
	return(SFerror = SFE_NSF);
    if(infoBmax >= SFDFLTBYTS && infoBmax < SFInfoSize(snd) )
	hedSiz = sizeof(SFSTRUCT)-SFDFLTBYTS + infoBmax;
    else
	hedSiz = snd->headSize;
    if(fseek(file, (long)0, SEEK_SET) != 0 )	/* move to start of file */
	return(SFerror = SFE_WRERR);
    if( (num = fwrite((void *)snd, (size_t)1, (size_t)hedSiz,
			 file))< (size_t)hedSiz)
	{
    /*	fprintf(stderr, "SFWrH: tried %ld managed %ld\n",
		hedSiz, (long)num);	*/
	return(SFerror = SFE_WRERR);
	}
    return(SFerror = SFE_OK);
    }

static long SFHeDaLen(FILE *file)
    /* return data space used for header in file - see SFCloseWrHdr */
    {
    long hlen = 0;

    /* assume file ptr is at last data written */
    if(fseek(file, (long) &((SFSTRUCT *) 0)->headBsize, SEEK_SET)
	!= 0 )	    /* need to re-read infoBsize */
	{
	fread((void *)&hlen, (size_t)sizeof(long), (size_t)1,file);
	}
    else
	hlen = (long)sizeof(SFSTRUCT);	/* default */
    return(hlen);
    }

static long SFLastSampTell(file)	/* return last valid ftell for smps */
    FILE	*file;
    {
    return (0L);	/* punt on this for here - just use EOF */
    }

static void FixupSamples(ptr, format, nitems, flag)
    void  *ptr;
    int   format;
    long  nitems;
    int   flag;		/* READ_FLAG or WRITE_FLAG indicates direction */
    {
    }	/* always machine order */
